@@ -10,24 +10,28 @@ Audio::Libshout - binding to the xiph.org libshout icecast source client
 
 =begin code
 
-    use Audio::Libshout;
+use v6;
 
+use Audio::Libshout;
+
+sub MAIN(IO(Str) $in-file) {
     my $shout = Audio::Libshout.new(password => 'hackme', mount => '/foo', format => Audio::Libshout::Format::MP3);
-    my $fh = @*ARGS[0].IO.open(:bin);
+    my $fh = $in-file.open(:bin);
     my $channel = $shout.send-channel;
 
     while not $fh.eof {
-	    my $buf = $fh.read(4096);
-	    $channel.send($buf);
+        my $buf = $fh.read(4096);
+        $channel.send($buf);
     }
 
     $fh.close;
     $channel.close;
     $shout.close;
+}
 
 =end code
 
-See also the files in the C<examples> directory
+See also the files in the L<examples|examples> directory
 
 =head1 DESCRIPTION
 
@@ -84,10 +88,20 @@ before calling C<open>.  If a connection cannot be made to the server or
 the server refuses the connection (e.g. due to an authentication failure,)
 a L<X::ShoutError> will be thrown.
 
+It should be noted that the behaviour when there is a failure after the
+initial connection to the server has been made  (such as an authentication
+problem,) may differ between versions of C<libshout>:  v2.2.2 will allow
+the connections parameters (e.g. the password,) to be changed and the
+C<open> retried using the same instance, however later versions (I have tested
+with v2.4.3,) may fail again with "already connected" and you will need
+to create a new C<Audio::Libshout> instance with the revised parameters
+to try again.
+
 This will be called for you the first time that C<send> is called or
 the first data is sent to the C<send-channel> however you may wish to
 call it early in order to detect and rectify any problems.
 
+
 =head2 method close
 
     method close()
@@ -119,7 +133,7 @@ a C<CArray[uint8]> and the second an C<Int> which is the number of
 bytes in the array, this reflects the return value of the C<encode>
 methods of L<Audio::Encode::LameMP3> with the C<:raw> adverb. This
 is intended to reduce the marshalling required when there is no
-need to have the data in perl space.
+need to have the data in Raku space.
 
 If you don't want to be concerned with the synchronisation issues then you
 should consider the asynchronous interface provided by C<send-channel>.
@@ -139,7 +153,7 @@ If you don't want to be concerned with the synchronisation issues then you
 should consider the asynchronous interface provided by C<send-channel>.
 
 =head2 method send-channel
-    
+
     multi method send-channel() returns Channel
     multi method send-channel(Channel $channel) returns Channel
 
@@ -197,6 +211,19 @@ is C<MP3>.
 It will throw a L<X::ShoutError> if an error is encountered while setting
 the metadata.
 
+=head2 dispose
+
+    method dispose()
+
+This will free the resources allocated by C<libshout> and de-initialise
+its internal global data. Once it has been called you should not attempt
+to send any further data to the stream and you should create a new instance
+if you need one.
+
+You may want to call this if you have a long-lived program which makes
+many relatively short-lived connections to a server.  However it may be
+simpler to C<open> and C<close> a single object as required.
+
 =head2 libshout-version
 
 This will return a L<Version> object that represents the version of the
@@ -211,34 +238,35 @@ opened. Setting a parameter that doesn't make sense given the state of
 the stream or already set parameters will result in a L<X::ShoutError>
 being thrown.
 
-=head2 host 
+=head2 host
 
-The hostname or IP address of the streaming server.  The default is 'localhost'.
+The hostname or IP address of the streaming server.  The default is
+'localhost'.
 
-=head2 port 
+=head2 port
 
 The port number on which the server is listening.  The default is 8000
 
-=head2 user 
+=head2 user
 
-The username that is used to authenticate against the server, the default is 'source'.
-If the C<protocol> is set to C<ICY> then setting this makes no sense as 'source' is
-always used.
+The username that is used to authenticate against the server, the default
+is 'source'.  If the C<protocol> is set to C<ICY> then setting this
+makes no sense as 'source' is always used.
 
-=head2 password 
+=head2 password
 
-The password that is used to authenticate with the server.  There is no default and this
-must be provided before connecting.
+The password that is used to authenticate with the server.  There is no
+default and this must be provided before connecting.
 
-=head2 protocol 
+=head2 protocol
 
-A value of the C<enum> L<Audio::Libshout::Protocol> indicating which protocol should be used
-to communicate with the server:
+A value of the C<enum> L<Audio::Libshout::Protocol> indicating which
+protocol should be used to communicate with the server:
 
 =item HTTP
 
-This is the Icecast v2 protocol and the default, this should be used unless there is a 
-compelling reason to do otherwise.
+This is the Icecast v2 protocol and the default, this should be used
+unless there is a compelling reason to do otherwise.
 
 =item XAUDIOCAST
 
@@ -246,71 +274,80 @@ The protocol used by version 1 of Icecast.
 
 =item ICY
 
-The Shoutcast protocol. If this is used then there are certain constraints on other parameters,
-it should only be used if you are using an actual Shoutcast server.
+The Shoutcast protocol. If this is used then there are certain constraints
+on other parameters, it should only be used if you are using an actual
+Shoutcast server.
 
-=head2 format 
+=head2 format
 
-The encoding format that is to be sent as a value of the C<enum> L<Audio::Libshout::Format> - the
-default is C<Ogg>.  No transcoding is done so the data to be sent to the server must be in the
-configured format - the result of setting a different format to the actual format of the data is that
-you will get an unreadable stream. Later versions of C<libshout> might provide for further formats.
+The encoding format that is to be sent as a value of the C<enum>
+L<Audio::Libshout::Format> - the default is C<Ogg>.  No transcoding
+is done so the data to be sent to the server must be in the configured
+format - the result of setting a different format to the actual format
+of the data is that you will get an unreadable stream. Later versions
+of C<libshout> might provide for further formats.
 
 =item MP3
 
 =item Ogg
 
-=head2 mount 
+=head2 mount
 
-The "mount point" (i.e. the path part ) on the server that represents this stream. There is no
-default. The C<ICY> protocol does not support setting this. On setting this will be "normalised"
-with a leading '/' (e.g. setting "stream" will return "/stream".)
+The "mount point" (i.e. the path part ) on the server that represents
+this stream. There is no default. The C<ICY> protocol does not support
+setting this. On setting this will be "normalised" with a leading '/'
+(e.g. setting "stream" will return "/stream".)
 
-=head2 dumpfile 
+=head2 dumpfile
 
-This can be set to cause an archive of the stream to be made on the server with the provided name.
-The server may not support this (or may configured to allow it.)  The resulting file will be at
-least as large as the streamed data and if the server runs out of disk space it may interrupt the
-stream, so think carefully before using this.
+This can be set to cause an archive of the stream to be made on the
+server with the provided name.  The server may not support this (or may
+configured to allow it.)  The resulting file will be at least as large
+as the streamed data and if the server runs out of disk space it may
+interrupt the stream, so think carefully before using this.
 
-=head2 agent 
+=head2 agent
 
-This is the UserAgent header that is sent for the C<HTTP> protocol.  The default is "libshout/$version".
+This is the UserAgent header that is sent for the C<HTTP> protocol.
+The default is "libshout/$version".
 
-=head2 public 
+=head2 public
 
-This is a L<Bool> that indicates whether the server should list the stream in any directory services that
-it has configured. The default is C<False>.
+This is a L<Bool> that indicates whether the server should list the stream
+in any directory services that it has configured. The default is C<False>.
 
 
-=head2 name 
+=head2 name
 
-The stream name that should be used in a directory listing. This may also be passed on to connected clients.
-There is no default.
+The stream name that should be used in a directory listing. This may
+also be passed on to connected clients.  There is no default.
 
-=head2 url 
+=head2 url
 
-The stream URL that should be used in a directory listing. This may also be passed on to connected clients.
-There is no default.
+The stream URL that should be used in a directory listing. This may also
+be passed on to connected clients.  There is no default.
 
-=head2 genre 
+=head2 genre
 
-The genre of the stream that should be used in a directory listing. This may also be passed on to connected clients.
-There is no default.
+The genre of the stream that should be used in a directory listing. This
+may also be passed on to connected clients.  There is no default.
 
 =head2 description
 
-A description of the stream that should be used in a directory listing. This may also be passed on to connected clients.
-There is no default.
+A description of the stream that should be used in a directory
+listing. This may also be passed on to connected clients.  There is
+no default.
 
 =end pod
 
-class Audio::Libshout:ver<v0.0.7>:auth<github:jonathanstowe> {
+class Audio::Libshout:ver<0.0.13>:auth<github:jonathanstowe>:api<1.0> {
     use NativeCall;
     use AccessorFacade;
 
+    my constant LIB = [ 'shout', v3 ];
+
     # encapsulate (bytes, count) in a type safe way
-    subset RawEncode of Array where  ($_.elems == 2 ) && ($_[0] ~~ CArray[uint8]) && ($_[1] ~~ Int);
+    subset RawEncode of Array where  ($_.elems == 2 ) && ($_[0] ~~ CArray) && ($_[1] ~~ Int);
 
     enum Error ( Success => 0 , Insane => -1 , Noconnect => -2 , Nologin => -3 , Socket => -4 , Malloc => -5, Meta => -6, Connected => -7, Unconnected => -8, Unsupported => -9, Busy => -10 );
     enum Format ( Ogg => 0, MP3 => 1 );
@@ -362,21 +399,25 @@ class Audio::Libshout:ver<v0.0.7>:auth<github:jonathanstowe> {
     }
 
     class Metadata is repr('CPointer') {
-        sub shout_metadata_new() returns Metadata is native('shout',v3) { * }
+        sub shout_metadata_new( --> Metadata ) is native(LIB) { * }
 
         method new() {
             shout_metadata_new();
         }
 
-        sub shout_metadata_free(Metadata) is native('shout',v3) { * }
+        sub shout_metadata_free(Metadata) is native(LIB) { * }
 
         method DESTROY() {
+            self.dispose;
+        }
+
+        method dispose() {
             shout_metadata_free(self);
         }
 
-        sub shout_metadata_add(Metadata, Str, Str ) returns int32 is native('shout',v3) { * }
+        sub shout_metadata_add(Metadata, Str, Str  --> int32 ) is native(LIB) { * }
 
-        method add(Str $key is rw, Str $value is rw) returns Error {
+        method add(Str $key is copy, Str $value is copy --> Error ) {
             explicitly-manage($key);
             explicitly-manage($value);
             my $rc = shout_metadata_add(self, $key, $value);
@@ -387,9 +428,9 @@ class Audio::Libshout:ver<v0.0.7>:auth<github:jonathanstowe> {
 
     class Shout is repr('CPointer') {
 
-        sub shout_new() returns Shout is native('shout',v3) { * }
+        sub shout_new( --> Shout ) is native(LIB) { * }
 
-        method new(Shout:U: *%attribs) returns Shout {
+        method new(Shout:U: *%attribs --> Shout ) {
             my $shout = shout_new();
 
             for %attribs.kv -> $attrib, $value {
@@ -400,40 +441,45 @@ class Audio::Libshout:ver<v0.0.7>:auth<github:jonathanstowe> {
             $shout;
         }
 
-        sub shout_free(Shout) returns int32 is native('shout',v3) { * }
+        sub shout_free(Shout --> int32 ) is native(LIB) { * }
 
         submethod DESTROY() {
+            self.dispose;
+        }
+
+        method dispose() {
             shout_free(self);
         }
 
-        sub shout_open(Shout) returns int32 is native('shout',v3) { * }
+        sub shout_open(Shout) returns int32 is native(LIB) { * }
 
-        method open() returns Error {
+        method open( --> Error ) {
             my $rc = shout_open(self);
             Error($rc);
         }
 
-        sub shout_close(Shout) returns int32 is native('shout',v3) { * }
+        sub shout_close(Shout --> int32 ) is native(LIB) { * }
 
-        method close() returns Error {
+        method close( --> Error ) {
             my $rc = shout_close(self);
             Error($rc);
         }
-        
-        sub shout_send(Shout, CArray[uint8], int32) returns int32 is native('shout',v3) { * }
 
-        multi method send(CArray[uint8] $buf, Int $elems) returns Error {
+        sub shout_send(Shout, CArray[uint8], int32 --> int32 ) is native(LIB) { * }
+
+        proto method send(|c) { * }
+        multi method send(CArray $buf, Int $elems --> Error ) {
             my $rc = shout_send(self, $buf, $elems);
             Error($rc);
         }
 
-        multi method send(Buf $buf) returns Error {
+        multi method send(Buf $buf --> Error ) {
             my $carray = CArray[uint8].new;
             $carray[$_] = $buf[$_] for ^($buf.elems);
             self.send($carray, $buf.elems);
         }
 
-        sub shout_sync(Shout) is native('shout',v3) { * }
+        sub shout_sync(Shout) is native(LIB) { * }
 
         method sync() {
             shout_sync(self);
@@ -457,99 +503,99 @@ class Audio::Libshout:ver<v0.0.7>:auth<github:jonathanstowe> {
 
         # "attributes"
         #
-        sub shout_set_host(Shout, Str) returns int32 is native('shout',v3) { * }
-        sub shout_get_host(Shout) returns Str is native('shout',v3) { * }
+        sub shout_set_host(Shout, Str --> int32 ) is native(LIB) { * }
+        sub shout_get_host(Shout --> Str ) is native(LIB) { * }
 
-        method host() returns Str is rw is accessor-facade(&shout_get_host, &shout_set_host, &manage, &check) { }
+        method host( --> Str ) is rw is accessor-facade(&shout_get_host, &shout_set_host, &manage, &check) { }
 
-        sub shout_set_port(Shout, int32) returns int32 is native('shout',v3) { * }
-        sub shout_get_port(Shout) returns int32 is native('shout',v3) { * }
+        sub shout_set_port(Shout, int32 --> int32 ) is native(LIB) { * }
+        sub shout_get_port(Shout --> int32 ) is native(LIB) { * }
 
-        method port() returns Int is rw is accessor-facade(&shout_get_port, &shout_set_port, Code, &check) { }
+        method port( --> Int ) is rw is accessor-facade(&shout_get_port, &shout_set_port, Code, &check) { }
 
-        sub shout_set_user(Shout, Str) returns int32 is native('shout',v3) { * }
-        sub shout_get_user(Shout) returns Str is native('shout',v3) { * }
+        sub shout_set_user(Shout, Str --> int32 ) is native(LIB) { * }
+        sub shout_get_user(Shout --> Str ) is native(LIB) { * }
 
-        method user() returns Str is rw is accessor-facade(&shout_get_user, &shout_set_user, &manage, &check) { }
+        method user( --> Str ) is rw is accessor-facade(&shout_get_user, &shout_set_user, &manage, &check) { }
 
-        sub shout_set_password(Shout, Str) returns int32 is native('shout',v3) { * }
-        sub shout_get_password(Shout) returns Str is native('shout',v3) { * }
+        sub shout_set_password(Shout, Str --> int32 ) is native(LIB) { * }
+        sub shout_get_password(Shout --> Str ) is native(LIB) { * }
 
-        method password() returns Str is rw is accessor-facade(&shout_get_password, &shout_set_password, &manage, &check) { }
+        method password( --> Str ) is rw is accessor-facade(&shout_get_password, &shout_set_password, &manage, &check) { }
 
-        sub shout_get_protocol(Shout) returns int32 is native('shout',v3) { * }
-        sub shout_set_protocol(Shout, int32) returns int32 is native('shout',v3) { * }
+        sub shout_get_protocol(Shout --> int32 ) is native(LIB) { * }
+        sub shout_set_protocol(Shout, int32 --> int32 ) is native(LIB) { * }
 
-        method protocol() returns Protocol is rw is accessor-facade(&shout_get_protocol, &shout_set_protocol, Code, &check) { }
+        method protocol( --> Protocol ) is rw is accessor-facade(&shout_get_protocol, &shout_set_protocol, Code, &check) { }
 
-        sub shout_get_format(Shout) returns int32 is native('shout',v3) { * }
-        sub shout_set_format(Shout, int32) returns int32 is native('shout',v3) { * }
+        sub shout_get_format(Shout --> int32 ) is native(LIB) { * }
+        sub shout_set_format(Shout, int32 --> int32 ) is native(LIB) { * }
 
-        method format() returns Format is rw is accessor-facade(&shout_get_format, &shout_set_format, Code, &check) { }
+        method format( --> Format ) is rw is accessor-facade(&shout_get_format, &shout_set_format, Code, &check) { }
 
-        sub shout_get_mount(Shout) returns Str is native('shout',v3) { * }
-        sub shout_set_mount(Shout, Str) returns int32 is native('shout',v3) { * }
+        sub shout_get_mount(Shout --> Str ) is native(LIB) { * }
+        sub shout_set_mount(Shout, Str --> int32 ) is native(LIB) { * }
 
-        method mount() returns Str is rw is accessor-facade(&shout_get_mount, &shout_set_mount, &manage, &check) { }
+        method mount( --> Str ) is rw is accessor-facade(&shout_get_mount, &shout_set_mount, &manage, &check) { }
 
-        sub shout_get_dumpfile(Shout) returns Str is native('shout',v3) { * }
-        sub shout_set_dumpfile(Shout, Str ) returns int32 is native('shout',v3) { * }
+        sub shout_get_dumpfile(Shout --> Str ) is native(LIB) { * }
+        sub shout_set_dumpfile(Shout, Str --> int32 ) is native(LIB) { * }
 
-        method dumpfile() returns Str is rw is accessor-facade(&shout_get_dumpfile, &shout_set_dumpfile, &manage, &check ) { }
+        method dumpfile( --> Str ) is rw is accessor-facade(&shout_get_dumpfile, &shout_set_dumpfile, &manage, &check ) { }
 
-        sub shout_get_agent(Shout) returns Str is native('shout',v3) { * }
-        sub shout_set_agent(Shout, Str) returns int32 is native('shout',v3) { * }
+        sub shout_get_agent(Shout --> Str ) is native(LIB) { * }
+        sub shout_set_agent(Shout, Str --> int32 ) is native(LIB) { * }
 
-        method agent() returns Str is rw is accessor-facade(&shout_get_agent, &shout_set_agent, &manage, &check) { }
+        method agent(--> Str ) is rw is accessor-facade(&shout_get_agent, &shout_set_agent, &manage, &check) { }
 
         # Directory parameters
-        sub shout_get_public(Shout) returns int32 is native('shout',v3) { * }
-        sub shout_set_public(Shout, int32) returns int32 is native('shout',v3) { * }
+        sub shout_get_public(Shout --> int32 ) is native(LIB) { * }
+        sub shout_set_public(Shout, int32 --> int32 ) is native(LIB) { * }
 
-        method public returns Bool is rw is accessor-facade(&shout_get_public, &shout_set_public, Code, &check) { }
+        method public( --> Bool ) is rw is accessor-facade(&shout_get_public, &shout_set_public, Code, &check) { }
 
-        sub shout_get_name(Shout) returns Str is native('shout',v3) { * }
-        sub shout_set_name(Shout, Str) returns int32 is native('shout',v3) { * }
+        sub shout_get_name(Shout --> Str ) is native(LIB) { * }
+        sub shout_set_name(Shout, Str --> int32 ) is native(LIB) { * }
 
-        method name() returns Str is rw is accessor-facade(&shout_get_name, &shout_set_name, &manage, &check) { }
+        method name( --> Str ) is rw is accessor-facade(&shout_get_name, &shout_set_name, &manage, &check) { }
 
-        sub shout_get_url(Shout) returns Str is native('shout',v3) { * }
-        sub shout_set_url(Shout, Str) returns int32 is native('shout',v3) { * }
+        sub shout_get_url(Shout --> Str ) is native(LIB) { * }
+        sub shout_set_url(Shout, Str --> int32 ) is native(LIB) { * }
 
-        method url() returns Str is rw is accessor-facade(&shout_get_url, &shout_set_url, &manage, &check) { }
+        method url( --> Str ) is rw is accessor-facade(&shout_get_url, &shout_set_url, &manage, &check) { }
 
-        sub shout_get_genre(Shout) returns Str is native('shout',v3) { * }
-        sub shout_set_genre(Shout, Str) returns int32 is native('shout',v3) { * }
+        sub shout_get_genre(Shout --> Str ) is native(LIB) { * }
+        sub shout_set_genre(Shout, Str --> int32 ) is native(LIB) { * }
 
-        method genre() returns Str is rw is accessor-facade(&shout_get_genre, &shout_set_genre, &manage, &check) { }
+        method genre( --> Str ) is rw is accessor-facade(&shout_get_genre, &shout_set_genre, &manage, &check) { }
 
-        sub shout_get_description(Shout) returns Str is native('shout',v3) { * }
-        sub shout_set_description(Shout, Str) returns int32 is native('shout',v3) { * }
+        sub shout_get_description(Shout --> Str ) is native(LIB) { * }
+        sub shout_set_description(Shout, Str --> int32 ) is native(LIB) { * }
 
-        method description() returns Str is rw is accessor-facade(&shout_get_description, &shout_set_description, &manage, &check) { }
+        method description( --> Str ) is rw is accessor-facade(&shout_get_description, &shout_set_description, &manage, &check) { }
 
         # Not making a method for these quite yet.
-        sub shout_get_audio_info(Shout, Str) returns Str is native('shout',v3) { * }
-        sub shout_set_audio_info(Shout, Str, Str) returns Str is native('shout',v3) { * }
+        sub shout_get_audio_info(Shout, Str --> Str ) is native(LIB) { * }
+        sub shout_set_audio_info(Shout, Str, Str --> Str ) is native(LIB) { * }
 
         # Set the metadata on this instance.
 
-        sub shout_set_metadata(Shout, Metadata) returns int32 is native('shout',v3) { * }
+        sub shout_set_metadata(Shout, Metadata --> int32 ) is native(LIB) { * }
 
-        method set-metadata(Metadata $meta) returns Error {
+        method set-metadata(Metadata $meta --> Error ) {
             my $rc = shout_set_metadata(self, $meta);
             Error($rc);
         }
 
-        sub shout_get_error(Shout) returns Str is native('shout',v3) { * };
+        sub shout_get_error(Shout --> Str ) is native(LIB) { * };
 
-        method last-error() returns Str {
+        method last-error( --> Str ) {
             shout_get_error(self);
         }
 
-        sub shout_get_errno(Shout) returns int32 is native('shout',v3) { * };
+        sub shout_get_errno(Shout --> int32 ) is native(LIB) { * };
 
-        method last-error-number() returns Error {
+        method last-error-number( --> Error ) {
             my $rc = shout_get_errno(self);
             Error($rc);
         }
@@ -558,7 +604,7 @@ class Audio::Libshout:ver<v0.0.7>:auth<github:jonathanstowe> {
     class Initialiser {
         has Int $!initialisers = 0;
 
-        sub shout_init() is native('shout',v3) { * }
+        sub shout_init() is native(LIB) { * }
 
         method init() {
             if $!initialisers == 0 {
@@ -567,7 +613,7 @@ class Audio::Libshout:ver<v0.0.7>:auth<github:jonathanstowe> {
             ++$!initialisers;
         }
 
-        sub shout_shutdown() is native('shout',v3) { * }
+        sub shout_shutdown() is native(LIB) { * }
 
         method shutdown() {
             --$!initialisers;
@@ -597,6 +643,7 @@ class Audio::Libshout:ver<v0.0.7>:auth<github:jonathanstowe> {
         }
     }
 
+    proto method send(|c) { * }
     multi method send(Buf $buff) {
         if not $!opened {
             self.open()
@@ -607,7 +654,7 @@ class Audio::Libshout:ver<v0.0.7>:auth<github:jonathanstowe> {
         }
     }
 
-    multi method send(CArray[uint8] $buff, Int $bytes) {
+    multi method send(CArray $buff, Int $bytes) {
         if not $!opened {
             self.open()
         }
@@ -621,19 +668,36 @@ class Audio::Libshout:ver<v0.0.7>:auth<github:jonathanstowe> {
         self.send($raw[0], $raw[1]);
     }
 
-    multi method send-channel(Audio::Libshout $self:) returns Channel {
+    proto method send-channel(|c) { * }
+
+    multi method send-channel(Audio::Libshout $self: --> Channel ) {
         my $channel = Channel.new;
         $self.send-channel($channel);
     }
 
+    class X::ChannelError is Exception {
+        has Exception $.inner;
+        method message() {
+            "An error occurred in the send-channel : '{ $!inner.message }'";
+        }
+    }
 
-    multi method send-channel(Audio::Libshout $self: Channel $channel) returns Channel {
-        $!helper-promise = start {
+    multi method send-channel(Audio::Libshout $self: Channel $channel --> Channel ) {
+        sub send() {
             for $channel.list -> $item {
-                $self.send($item);
-                $self.sync;
+                    CATCH {
+                        default {
+                            $self.close;
+                            X::ChannelError.new(inner => $_).throw;
+                        }
+                    }
+                    LAST {
+                    }
+                    $self.send($item);
+                    $self.sync;
             }
         }
+        $!helper-promise = Promise.start(&send, scheduler => ThreadPoolScheduler.new);
         $channel;
     }
 
@@ -656,6 +720,8 @@ class Audio::Libshout:ver<v0.0.7>:auth<github:jonathanstowe> {
         }
     }
 
+    proto method add-metadata(|c) { * }
+
     multi method add-metadata(Str $key, Str $value) {
         my $rc = $!metadata.add($key, $value);
         if $rc !~~ Success {
@@ -680,8 +746,8 @@ class Audio::Libshout:ver<v0.0.7>:auth<github:jonathanstowe> {
     }
 
 
-    sub shout_version(int32, int32, int32) returns Str is native('shout',v3) { * }
-    method libshout-version() returns Version {
+    sub shout_version(int32, int32, int32 --> Str ) is native(LIB) { * }
+    method libshout-version( --> Version ) {
         my int32 $major;
         my int32 $minor;
         my int32 $patch;
@@ -698,12 +764,20 @@ class Audio::Libshout:ver<v0.0.7>:auth<github:jonathanstowe> {
         $!metadata = Metadata.new;
     }
 
-    submethod DESTROY() {
-        $!shout = Shout;
-        $!metadata = Metadata;
+    method dispose(Audio::Libshout:D:) {
+        if $!metadata.defined {
+            $!metadata = Metadata;
+        }
+        if $!shout.defined {
+            $!shout.dispose;
+            $!shout = Shout;
+        }
         $initialiser.shutdown();
     }
 
+    submethod DESTROY() {
+        self.dispose;
+    }
 }
 
-# vim: expandtab shiftwidth=4 ft=perl6
+# vim: expandtab shiftwidth=4 ft=raku
